%
%
% Title: Filtered Correlation and Allowed Frequency Spectra in Dynamic 
%        Functional Connectivity
% Journal of Neuroscience Methods
% JNEUMETH-D-20-00071
%
% Figure 1
%
% Coded by : Victor M. Vergara, PhD
%			 2020-June- 9: Submitted Revision R1
%			 2020-June-29: Minor change mostly comments
%
clear;clc;close all;
fprintf(['\nFiltered Correlation and \nAllowed Frequency',	...
         ' Spectra in \nDynamic Functional Connectivity\n']);
fprintf('\nJournal of Neuroscience Methods\n');
fprintf('\n\nVictor M. Vergara and Vince D. Calhoun\n');
fprintf(['\n\nFigure1: \n\tSimulation of fMRI signals and \n\t', ...
		'estimation of time varying correlation using x(t) * y(t)\n\n'])

%% CONFIGURATION: time courses x(t) and y(t)
% Configuring time variables
FontSz = 16;	% Just the font size for figures
TR =  2;	% [sec] simulated MRI TR
timeL = 600;	% [sec] time length of the simulation
nIter = 1000;	% number of iterations
lowestFreq  = 0.05 ; % low  time course frequency in Hz
highestFreq = 0.10 ; % high time course frequency in Hz

% calculating variables from configuration
nTR = ceil(timeL/TR);	% how many TR to simulate
Fstep = 1/(TR*nTR);		% frequency step
t = TR*(0:nTR-1);	% time variable in seconds
F = lowestFreq:Fstep:highestFreq; % simulated cosine frequency in Hz
nF = length(F);	% number of simulated cosines
A = sqrt(2/nF)*ones(1,nF); % normalized cosine amplitudes

%% CONFIGURATION: Configure the filter
% low pass filter: remove the triangular artifact spectrum of x(t) * y(t)
% comment or uncomment to select the desired filter

% Infinite Impulse Response (IIR)
% butter_filter_order = 5;
% [bf,af] = butter(butter_filter_order,2*lowestFreq*TR);

% Finite Impulse Response (FIR)
nFilterTaps = 50;
af=1;bf=fir1(nFilterTaps,2*lowestFreq*TR);

%% PERFORMING SIMULATION

% Initialize memory for the Frequency Spectrums to Plot
FX = zeros(nIter,nTR);
FY = zeros(nIter,nTR);
FXY = zeros(nIter,nTR);
fFXY = zeros(nIter,nTR);

fprintf('Estimating mean frequency spectrum from %d iterations\n',nIter);
fprintf('\titeration number     ');
for ii=1:nIter
	x = zeros(1,nTR);
	y =	zeros(1,nTR);
	for f=1:nF	% Add each frequency
		p = 2*pi*rand(1,1);
		w = 2*pi*F(f);	% Frequency in rads/sec
		x = x + A(f)*cos(w*t + p );
		y = y + A(f)*cos(w*t + p + 1.4);
	end
	%% Calculate the XtimesY
	XY = x.*y;	
	%% Filter XY
	filteredXY = filter(bf,af,XY);
	%% ADd the spectrums
	FX(ii,:) =  abs(fft(x)).^2;
	FY(ii,:) =  abs(fft(y)).^2;
	FXY(ii,:) =  abs(fft(XY)).^2;
	fFXY(ii,:)= abs(fft(filteredXY)).^2;	

	fprintf('\b\b\b\b%04.0f',ii);
end
fprintf('\n\tsimulation finished\n')

%% plot the three signals x y and ground truth
fprintf('\nplotting Figure 1\n');
figure(1);
% plot 1 Time courses
subplot(3,2,1);
plot(t,x);
xlabel('Time sec','FontSize',FontSz);
ylabel('x_(_t_)','FontSize',FontSz);
title(['Band [ ',sprintf('%.3f',lowestFreq),' to ', ...
	sprintf('%.3f',highestFreq),' ] Hz']);


subplot(3,2,3);
plot(t,y,'r');
xlabel('Time sec','FontSize',FontSz);
ylabel('y_(_t_)','FontSize',FontSz);
title(['Band [ ',sprintf('%.3f',lowestFreq),' to ', ...
	sprintf('%.3f',highestFreq),' ] Hz']);
AX = axis();

subplot(3,2,5);
plot(t,XY);%,t,filteredXY);
xlabel('Time sec','FontSize',FontSz);
ylabel('y_(_t_) x_(_t_)','FontSize',FontSz);
title(['Band [ ',sprintf('%.3f',2*lowestFreq),' to ', ...
	sprintf('%.3f',2*highestFreq),' ] Hz']);
axis(AX);


%% plot the FRequency of C and Y
maxPlotFreq = 0.25;

freq_variable = (0:1:(nTR-1))/(nTR*TR);
freq_selection= 1:1:ceil(maxPlotFreq*nTR*TR);


subplot(3,2,2);
PSD = mean(FX)/(nTR^2);
f_plotPosNegRealFreqSpectrum( freq_variable(freq_selection),	...
		PSD(freq_selection),'b');
xlabel('Frequency Hz','FontSize',FontSz);
ylabel('X(f) PSD','FontSize',FontSz);
title(['Band [ ',sprintf('%.3f',lowestFreq),' to ', ...
	sprintf('%.3f',highestFreq),' ] Hz']);


subplot(3,2,4);

PSD = mean(FY)/(nTR^2);
f_plotPosNegRealFreqSpectrum( freq_variable(freq_selection),	...
		PSD(freq_selection),'r');
xlabel('Frequency Hz','FontSize',FontSz);
ylabel('Y(f) PSD','FontSize',FontSz);
title(['Band [ ',sprintf('%.3f',lowestFreq),' to ', ...
	sprintf('%.3f',highestFreq),' ] Hz']);

subplot(3,2,6);
PSD = mean(FXY)/(nTR^2);
f_plotPosNegRealFreqSpectrum( freq_variable(freq_selection),	...
		PSD(freq_selection),'k');
hold on;plot(0,PSD(1),'mo');hold off;
xlabel('Frequency Hz','FontSize',FontSz);
ylabel('[X*Y]_(_f_) PSD','FontSize',FontSz);
title(['Band [ ',sprintf('%.3f',2*lowestFreq),' to ', ...
	sprintf('%.3f',2*highestFreq),' ] Hz']);
axis tight

fprintf('done ...\n\n');
return


