%
%
% Title: Filtered Correlation and Allowed Frequency Spectra in Dynamic 
%        Functional Connectivity
% Journal of Neuroscience Methods
% JNEUMETH-D-20-00071
%
% Figure 3
%
% Coded by : Victor M. Vergara, PhD
%			 2020-June- 9: Submitted Revision R1
%			 2020-June-29: Minor change mostly comments
%
%
%
clear;clc;close all;
fprintf(['\nFiltered Correlation and \nAllowed Frequency',	...
         ' Spectra in \nDynamic Functional Connectivity\n']);
fprintf('\nJournal of Neuroscience Methods\n');
fprintf('\n\nVictor M. Vergara and Vince D. Calhoun\n');
fprintf(['\n\nFigure3: \n\tSimulation of phase between\n\t', ...
		'simulated time varying signals x(t) and  y(t)\n\n'])
fprintf('\tx(t) and y(t) are sum of cosines temporal signals\n');
fprintf('\t\tx(t) = sum ( cos( w_i * t + angle1_i )\n');
fprintf('\t\ty(t) = sum ( cos( w_i * t + angle2_i + phase(t) )\n');
fprintf(['\tphase(t) is also time varying with several \n', ...
		 '\tfrequencies wp and phase-dephasings anglep\n']);
fprintf('\t\typhase(t) = sum ( cos( wp_i * t + anglep_i) )\n\n\n');

%% CONFIGURATION: Set the dynamic functional connectivity
%
% Uncomment the different configurations to simulate the different
% sub-figures a, b, c and d. By default Figure 3a is uncommented.
%
% Figure3a
fprintf('Simulating Figure 3a\n\n');
lowestFreq  = 0.02	; % time course lowest  time course frequency in Hz
highestFreq = 0.10	; % time course highest time course frequency in Hz
dFC_Frequencies = [0.01 0.005];% phase(x,y)(t) frequencies in Hz
dFC_Amplitudes = [0.4 0.4];    % phase(x,y)(t) amplitudes
dFC_Phases = [0 pi/5];         % phase(x,y)(t) dephasings in rads
nFilterTaps = 100;             % phase(x,y)(t) filter size in TR
cutoffFrquency = 0.02;         % phase(x,y)(t) low pass cut off freq

% Figure3b
% fprintf('Simulating Figure 3b\n\n');
% lowestFreq  = 0.02 ; % time course lowest  time course frequency in Hz
% highestFreq = 0.10 ; % time course highest time course frequency in Hz
% dFC_Frequencies = [0.01 0.1];% phase(x,y)(t) frequencies in Hz
% dFC_Amplitudes = [0.4 0.4];  % phase(x,y)(t) amplitudes
% dFC_Phases = [0 pi/5 2*pi/5];% phase(x,y)(t) dephasings in rads 
% nFilterTaps = 100;           % phase(x,y)(t) filter size in TR
% cutoffFrquency = 0.02;       % phase(x,y)(t) low pass cut off freq

%Figure 3c
% fprintf('Simulating Figure 3c\n\n');
% lowestFreq  = 0.02 ; % time course lowest  time course frequency in Hz
% highestFreq = 0.10 ; % time course highest time course frequency in Hz
% dFC_Frequencies = [0.01 0.1];% phase(x,y)(t) frequencies in Hz
% dFC_Amplitudes = [0.4 0.4];  % phase(x,y)(t) amplitudes
% dFC_Phases = [0 pi/5];       % phase(x,y)(t) dephasings in rads
% nFilterTaps = 100;           % phase(x,y)(t) filter size in TR
% cutoffFrquency = 0.12;       % phase(x,y)(t) low pass cut off freq

%Figure 3d
% fprintf('Simulating Figure 3d\n\n');
% lowestFreq  = 0.07 ; % time course lowest  time course frequency in Hz
% highestFreq = 0.10 ; % time course highest time course frequency in Hz
% dFC_Frequencies = [0.01 0.1];% phase(x,y)(t) frequencies in Hz
% dFC_Amplitudes = [0.4 0.4];  % phase(x,y)(t) amplitudes
% dFC_Phases = [0 pi/5];       % phase(x,y)(t) dephasings in rads
% nFilterTaps = 100;           % phase(x,y)(t) filter size in TR
% cutoffFrquency = 0.12;       % phase(x,y)(t) low pass cut off freq


%% CONFIGURATION: Setup the simulation 
% Configuring variables
FontSz = 22; % Just the font size for figures
axisFontSz = 14; % Just the font size for figure axes
TR =  2;% [sec] simulated MRI TR
timeL = 600;% [sec] time length of the simulation
nIter = 100;	% number of iterations

% variables calculated from configuration
nTR = ceil(timeL/TR);	% How many TR to simulate
t = TR*(0:nTR-1);	% time variable in seconds
Fstep = 1/(TR*nTR); % frequency step
% Fstep = 0.01;
F  = lowestFreq:Fstep:highestFreq; % simulated cosine frequency in Hz
nF = length(F);	% Number of Frequencies
A = sqrt(2/nF)*ones(1,nF); % makes x and y have var = 1

%% ESTIMATE Dynamic Connectivity dc(x,y)(t) to Simulated 
dFC = zeros(length(dFC_Frequencies),length(t));
for kk = 1:length(dFC_Frequencies)
	dFC(kk,:) = dFC_Amplitudes(kk)*cos(2*pi*dFC_Frequencies(kk)*t + ...
		dFC_Phases(kk));
end
CC = mean(dFC); % Simulated Ground Truth dc(x,y)(t) = sum of cosines  

%% Configuration: Set the FIR filter
af=1;
bf=fir1(nFilterTaps,2*cutoffFrquency*TR);

%% PERFORMING SIMULATION
FXY = zeros(nIter,nTR);
FfiltXY = zeros(nIter,nTR);
corrPlot = 0;
fprintf('Estimating mean frequency spectrum from %d iterations\n',nIter);
fprintf('\titeration number     ');
for ii=1:nIter
	%% Generate the time course signals x(t) and y(t)
	x = zeros(1,nTR);
	y =	zeros(1,nTR);
	for f=1:nF	% Add each frequency
		PP = 2*pi*rand(1,1);
		w = 2*pi*F(f);	% Frequency in rads/sec
		x = x + A(f)*cos(w*t + PP );
		y = y + A(f)*cos(w*t + PP + acos(CC) ); % phase(x,y) = acos(dc) 
	end
	%% Calculate XY =  x(t) times y(t) 
	XY = x.*y;
	XY = XY - mean(XY);
	
	%% Calculate the Filtered XY
	filtXY = filter(bf,af,XY);
	filtXY = filtXY - mean(filtXY);
	
	%% Add the spectrums
	FXY(ii,:) =  abs(fft(XY)).^2;
	FfiltXY(ii,:)= abs(fft(filtXY)).^2; 
	
	%% This section selects one of the iterations to plot the time courses
	% just find the correlation without the problem of filter settling time
	% plot(CC(floor(nFilterTaps/2) + (1:nTR-nFilterTaps))');
	% hold on;plot( filtXY(1:nTR-nFilterTaps));hold off;
	ccp = abs(corr(CC(floor(nFilterTaps/2) + ...
	      (1:nTR-nFilterTaps))',filtXY(1:nTR-nFilterTaps)'));
	% keep the max correlation results
	% corrPlot is the correlation between Ground Truth and the selected
	% XY(t) to plot
	if ccp > corrPlot
		XYp =XY;
		corrPlot = ccp;
	end
	
	fprintf('\b\b\b\b%04.0f',ii);
end
fprintf('\n\tsimulation finished\n')

%% PLOTTING
figure('Position',[64 64 1400 256]);
%% subplot 1 Ground Truth
subplot(1,4,1);
plot(t,CC,'k'); % veriable CC is the simulated groun truth dc(x,y)(t)
xlabel('Time sec','FontSize',FontSz);
ylabel('\theta_(_t_)','FontSize',FontSz);
set(gca,'FontSize',axisFontSz);
AX = axis();
AX = [ 0, 3+ max(t) - TR*(nFilterTaps/2), 1.5*AX(3:4)];
axis(AX);

%% subplot 2 time multiplication x(t) * y(t) 
subplot(1,4,2);
plot(t,XYp,'k');
xlabel('Time sec','FontSize',FontSz);
ylabel('x_(_t_) y_(_t_)','FontSize',FontSz);
axis([ AX(1:2) , 1.1*max(abs(XYp))*[-1 1]]);
set(gca,'FontSize',axisFontSz);

%% subplot 3 filtered version of x(t) * y(t)
subplot(1,4,3);
filxy = filter(bf,af,XYp);
filxy = filxy - mean(filxy);
plot(t - TR*(nFilterTaps/2),filxy,'k');
axis(AX);
xlabel('Time sec','FontSize',FontSz);
ylabel('FIR [ x_(_t_) y_(_t_) ]','FontSize',FontSz);
set(gca,'FontSize',axisFontSz);


%% Frequency PLot
% This plot estimate average frequency spectrums from nIter iterations
subplot(1,4,4);
fXY = (0:1:(nTR-1))/(nTR*TR);

maxfreqq = 0.25; % only for plotting purposes 
frqXY = 1:1:ceil(maxfreqq*nTR*TR);

%estimate the frequency spectrum of of the filter
filX = [bf,zeros(1,nTR - length(bf))];
filF = fft(filX);

% normalize freq spectrum
A= mean(FXY); 
B= mean(FfiltXY);
C = 0.99*abs(filF).^2; 
normAB = max([A,B]);
A = A./normAB;
B = B./normAB;
plot(fXY(frqXY),(A(frqXY)),'k',	...
	 fXY(frqXY),(B(frqXY)),'r--',	...
	 fXY(frqXY),(C(frqXY)),'b.-'	...
	 );
legend('XY','Filtered XY','FIR Low Pass')
xlabel('Frequency Hz','FontSize',FontSz);
ylabel('PSD','FontSize',FontSz);
axis([0 maxfreqq 0 1]);
set(gca,'FontSize',axisFontSz);


%% display the correlation between ground truth and FNC
fprintf('\n\nCorrelation GroundTruth vs. Filtered[XY] = %f\n\ndone\n', ...
	corrPlot);

